@@ -22,8 +22,11 @@
 #include "config.h"
 
 #ifdef USE_USBREDIR
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <usbredirhost.h>
+#ifdef USE_LZ4
+#include <lz4.h>
+#endif
 #ifdef USE_POLKIT
 #include "usb-acl-helper.h"
 #endif
@@ -32,6 +35,7 @@
 #include "usbutil.h"
 #endif
 
+#include "common/log.h"
 #include "spice-client.h"
 #include "spice-common.h"
 
@@ -51,6 +55,7 @@
 
 #ifdef USE_USBREDIR
 
+#define COMPRESS_THRESHOLD 1000
 #define SPICE_USBREDIR_CHANNEL_GET_PRIVATE(obj)                                  \
     (G_TYPE_INSTANCE_GET_PRIVATE((obj), SPICE_TYPE_USBREDIR_CHANNEL, SpiceUsbredirChannelPrivate))
 
@@ -79,6 +84,7 @@ struct _SpiceUsbredirChannelPrivate {
     SpiceUsbAclHelper *acl_helper;
 #endif
     GMutex device_connect_mutex;
+    SpiceUsbDeviceManager *usb_device_manager;
 };
 
 static void channel_set_handlers(SpiceChannelClass *klass);
@@ -188,6 +194,11 @@ static void spice_usbredir_channel_dispose(GObject *obj)
     SpiceUsbredirChannel *channel = SPICE_USBREDIR_CHANNEL(obj);
 
     spice_usbredir_channel_disconnect_device(channel);
+    /* This should have been set to NULL during device disconnection,
+     * but better not to leak it if this does not happen for some reason
+     */
+    g_warn_if_fail(channel->priv->usb_device_manager == NULL);
+    g_clear_object(&channel->priv->usb_device_manager);
 
     /* Chain up to the parent class */
     if (G_OBJECT_CLASS(spice_usbredir_channel_parent_class)->dispose)
@@ -205,7 +216,7 @@ static void spice_usbredir_channel_dispose(GObject *obj)
  * Now the last one may seem like an issue, since what will happen if
  * spice_usbredir_channel_open_acl_cb will run after finalization?
  *
- * This will never happens since the GSimpleAsyncResult created before we
+ * This will never happens since the GTask created before we
  * get into the STATE_WAITING_FOR_ACL_HELPER takes a reference to its
  * source object, which is our SpiceUsbredirChannel object, so
  * the finalize won't hapen until spice_usbredir_channel_open_acl_cb runs,
@@ -233,6 +244,7 @@ static void channel_set_handlers(SpiceChannelClass *klass)
 {
     static const spice_msg_handler handlers[] = {
         [ SPICE_MSG_SPICEVMC_DATA ] = usbredir_handle_msg,
+        [ SPICE_MSG_SPICEVMC_COMPRESSED_DATA ] = usbredir_handle_msg,
     };
 
     spice_channel_set_handlers(klass, handlers, G_N_ELEMENTS(handlers));
@@ -269,14 +281,19 @@ void spice_usbredir_channel_set_context(SpiceUsbredirChannel *channel,
 #if USBREDIR_VERSION >= 0x000701
     usbredirhost_set_buffered_output_size_cb(priv->host, usbredir_buffered_output_size_callback);
 #endif
+#ifdef USE_LZ4
+    spice_channel_set_capability(channel, SPICE_SPICEVMC_CAP_DATA_COMPRESS_LZ4);
+#endif
 }
 
 static gboolean spice_usbredir_channel_open_device(
     SpiceUsbredirChannel *channel, GError **err)
 {
     SpiceUsbredirChannelPrivate *priv = channel->priv;
+    SpiceSession *session;
     libusb_device_handle *handle = NULL;
     int rc, status;
+    SpiceUsbDeviceManager *manager;
 
     g_return_val_if_fail(priv->state == STATE_DISCONNECTED
 #ifdef USE_POLKIT
@@ -300,10 +317,12 @@ static gboolean spice_usbredir_channel_open_device(
         return FALSE;
     }
 
-    if (!spice_usb_device_manager_start_event_listening(
-            spice_usb_device_manager_get(
-                spice_channel_get_session(SPICE_CHANNEL(channel)), NULL),
-            err)) {
+    session = spice_channel_get_session(SPICE_CHANNEL(channel));
+    manager = spice_usb_device_manager_get(session, NULL);
+    g_return_val_if_fail(manager != NULL, FALSE);
+
+    priv->usb_device_manager = g_object_ref(manager);
+    if (!spice_usb_device_manager_start_event_listening(priv->usb_device_manager, err)) {
         usbredirhost_set_device(priv->host, NULL);
         return FALSE;
     }
@@ -481,12 +500,10 @@ void spice_usbredir_channel_disconnect_device(SpiceUsbredirChannel *channel)
          * usbredirhost_set_device NULL will interrupt the
          * libusb_handle_events call in the thread.
          */
-        {
-            SpiceSession *session = spice_channel_get_session(SPICE_CHANNEL(channel));
-            if (session != NULL)
-                spice_usb_device_manager_stop_event_listening(
-                    spice_usb_device_manager_get(session, NULL));
-        }
+        g_warn_if_fail(priv->usb_device_manager != NULL);
+        spice_usb_device_manager_stop_event_listening(priv->usb_device_manager);
+        g_clear_object(&priv->usb_device_manager);
+
         /* This also closes the libusb handle we passed from open_device */
         usbredirhost_set_device(priv->host, NULL);
         g_clear_pointer(&priv->device, libusb_unref_device);
@@ -518,6 +535,7 @@ gboolean spice_usbredir_channel_disconnect_device_finish(
     return g_task_propagate_boolean(G_TASK(res), err);
 }
 
+G_GNUC_INTERNAL
 void spice_usbredir_channel_disconnect_device_async(SpiceUsbredirChannel *channel,
                                                     GCancellable *cancellable,
                                                     GAsyncReadyCallback callback,
@@ -527,8 +545,17 @@ void spice_usbredir_channel_disconnect_device_async(SpiceUsbredirChannel *channe
 
     g_return_if_fail(channel != NULL);
     g_task_run_in_thread(task, _disconnect_device_thread);
+    g_object_unref(task);
 }
 
+#ifdef USE_LZ4
+static SpiceUsbDevice *
+spice_usbredir_channel_get_spice_usb_device(SpiceUsbredirChannel *channel)
+{
+    return channel->priv->spice_device;
+}
+#endif
+
 G_GNUC_INTERNAL
 libusb_device *spice_usbredir_channel_get_device(SpiceUsbredirChannel *channel)
 {
@@ -632,15 +659,83 @@ static void usbredir_free_write_cb_data(uint8_t *data, void *user_data)
     usbredirhost_free_write_buffer(priv->host, data);
 }
 
+#ifdef USE_LZ4
+static int try_write_compress_LZ4(SpiceUsbredirChannel *channel, uint8_t *data, int count)
+{
+    SpiceChannelPrivate *c;
+    SpiceMsgOut *msg_out_compressed;
+    int bound, compressed_data_count;
+    uint8_t *compressed_buf;
+    SpiceMsgCompressedData compressed_data_msg = {
+        .type = SPICE_DATA_COMPRESSION_TYPE_LZ4,
+        .uncompressed_size = count
+    };
+
+    c = SPICE_CHANNEL(channel)->priv;
+    if (g_socket_get_family(c->sock) == G_SOCKET_FAMILY_UNIX) {
+        /* AF_LOCAL socket - data will not be compressed */
+        return FALSE;
+    }
+    if (count <= COMPRESS_THRESHOLD) {
+        /* Not enough data to compress */
+        return FALSE;
+    }
+    if (!spice_channel_test_capability(SPICE_CHANNEL(channel),
+                                       SPICE_SPICEVMC_CAP_DATA_COMPRESS_LZ4)) {
+        /* No server compression capability - data will not be compressed */
+        return FALSE;
+    }
+    if (spice_usb_device_is_isochronous(spice_usbredir_channel_get_spice_usb_device(channel))) {
+        /* Don't compress - one of the device endpoints is isochronous */
+        return FALSE;
+    }
+    bound = LZ4_compressBound(count);
+    if (bound == 0) {
+        /* Invalid bound - data will not be compressed */
+        return FALSE;
+    }
+
+    compressed_buf = g_malloc(bound);
+    compressed_data_count = LZ4_compress_default((char*)data,
+                                                 (char*)compressed_buf,
+                                                 count,
+                                                 bound);
+    if (compressed_data_count > 0 && compressed_data_count < count) {
+        compressed_data_msg.compressed_data = compressed_buf;
+        msg_out_compressed = spice_msg_out_new(SPICE_CHANNEL(channel),
+                                               SPICE_MSGC_SPICEVMC_COMPRESSED_DATA);
+        msg_out_compressed->marshallers->msg_SpiceMsgCompressedData(msg_out_compressed->marshaller,
+                                                                    &compressed_data_msg);
+        spice_marshaller_add_by_ref_full(msg_out_compressed->marshaller,
+                                         compressed_data_msg.compressed_data,
+                                         compressed_data_count,
+                                         (spice_marshaller_item_free_func)g_free,
+                                         channel);
+        spice_msg_out_send(msg_out_compressed);
+        return TRUE;
+    }
+
+    /* if not - free & fallback to sending the message uncompressed */
+    g_free(compressed_buf);
+    return FALSE;
+}
+#endif
+
 static int usbredir_write_callback(void *user_data, uint8_t *data, int count)
 {
     SpiceUsbredirChannel *channel = user_data;
     SpiceMsgOut *msg_out;
 
+#ifdef USE_LZ4
+    if (try_write_compress_LZ4(channel, data, count)) {
+        usbredirhost_free_write_buffer(channel->priv->host, data);
+        return count;
+    }
+#endif
     msg_out = spice_msg_out_new(SPICE_CHANNEL(channel),
                                 SPICE_MSGC_SPICEVMC_DATA);
-    spice_marshaller_add_ref_full(msg_out->marshaller, data, count,
-                                  usbredir_free_write_cb_data, channel);
+    spice_marshaller_add_by_ref_full(msg_out->marshaller, data, count,
+                                     usbredir_free_write_cb_data, channel);
     spice_msg_out_send(msg_out);
 
     return count;
@@ -655,11 +750,13 @@ static void *usbredir_alloc_lock(void) {
     return mutex;
 }
 
+G_GNUC_INTERNAL
 void spice_usbredir_channel_lock(SpiceUsbredirChannel *channel)
 {
     g_mutex_lock(&channel->priv->device_connect_mutex);
 }
 
+G_GNUC_INTERNAL
 void spice_usbredir_channel_unlock(SpiceUsbredirChannel *channel)
 {
     g_mutex_unlock(&channel->priv->device_connect_mutex);
@@ -724,11 +821,49 @@ static void spice_usbredir_channel_up(SpiceChannel *c)
     usbredirhost_write_guest_data(priv->host);
 }
 
+static int try_handle_compressed_msg(SpiceMsgCompressedData *compressed_data_msg,
+                                     uint8_t **buf,
+                                     int *size) {
+    int decompressed_size = 0;
+    char *decompressed = NULL;
+
+    if (compressed_data_msg->uncompressed_size == 0) {
+        spice_warning("Invalid uncompressed_size");
+        return FALSE;
+    }
+
+    switch (compressed_data_msg->type) {
+#ifdef USE_LZ4
+    case SPICE_DATA_COMPRESSION_TYPE_LZ4:
+        decompressed = g_malloc(compressed_data_msg->uncompressed_size);
+        decompressed_size = LZ4_decompress_safe ((char*)compressed_data_msg->compressed_data,
+                                                 decompressed,
+                                                 compressed_data_msg->compressed_size,
+                                                 compressed_data_msg->uncompressed_size);
+        break;
+#endif
+    default:
+        spice_warning("Unknown Compression Type");
+        return FALSE;
+    }
+    if (decompressed_size != compressed_data_msg->uncompressed_size) {
+        spice_warning("Decompress Error decompressed_size=%d expected=%u",
+                      decompressed_size, compressed_data_msg->uncompressed_size);
+        g_free(decompressed);
+        return FALSE;
+    }
+
+    *size = decompressed_size;
+    *buf = (uint8_t*)decompressed;
+    return TRUE;
+
+}
+
 static void usbredir_handle_msg(SpiceChannel *c, SpiceMsgIn *in)
 {
     SpiceUsbredirChannel *channel = SPICE_USBREDIR_CHANNEL(c);
     SpiceUsbredirChannelPrivate *priv = channel->priv;
-    int r, size;
+    int r = 0, size;
     uint8_t *buf;
 
     g_return_if_fail(priv->host != NULL);
@@ -736,13 +871,23 @@ static void usbredir_handle_msg(SpiceChannel *c, SpiceMsgIn *in)
     /* No recursion allowed! */
     g_return_if_fail(priv->read_buf == NULL);
 
-    buf = spice_msg_in_raw(in, &size);
-    priv->read_buf = buf;
-    priv->read_buf_size = size;
+    if (spice_msg_in_type(in) == SPICE_MSG_SPICEVMC_COMPRESSED_DATA) {
+        SpiceMsgCompressedData *compressed_data_msg = spice_msg_in_parsed(in);
+        if (try_handle_compressed_msg(compressed_data_msg, &buf, &size)) {
+            priv->read_buf_size = size;
+            priv->read_buf = buf;
+        } else {
+            r = usbredirhost_read_parse_error;
+        }
+    } else { /* Regular SPICE_MSG_SPICEVMC_DATA msg */
+        buf = spice_msg_in_raw(in, &size);
+        priv->read_buf_size = size;
+        priv->read_buf = buf;
+    }
 
     spice_usbredir_channel_lock(channel);
-
-    r = usbredirhost_read_guest_data(priv->host);
+    if (r == 0)
+        r = usbredirhost_read_guest_data(priv->host);
     if (r != 0) {
         SpiceUsbDevice *spice_device = priv->spice_device;
         device_error_data err_data;
@@ -782,15 +927,16 @@ static void usbredir_handle_msg(SpiceChannel *c, SpiceMsgIn *in)
         err_data.caller = coroutine_self();
         err_data.spice_device = g_boxed_copy(spice_usb_device_get_type(), spice_device);
         err_data.error = err;
+        spice_usbredir_channel_unlock(channel);
         g_idle_add(device_error, &err_data);
         coroutine_yield(NULL);
 
         g_boxed_free(spice_usb_device_get_type(), err_data.spice_device);
 
         g_error_free(err);
+    } else {
+        spice_usbredir_channel_unlock(channel);
     }
-
-    spice_usbredir_channel_unlock(channel);
 }
 
 #endif /* USE_USBREDIR */
