@@ -18,13 +18,18 @@
 #ifndef __SPICE_CLIENT_CHANNEL_H__
 #define __SPICE_CLIENT_CHANNEL_H__
 
-G_BEGIN_DECLS
+#if !defined(__SPICE_CLIENT_H_INSIDE__) && !defined(SPICE_COMPILATION)
+#warning "Only <spice-client.h> can be included directly"
+#endif
 
 #include <gio/gio.h>
+
 #include "spice-types.h"
 #include "spice-glib-enums.h"
 #include "spice-util.h"
 
+G_BEGIN_DECLS
+
 #define SPICE_TYPE_CHANNEL            (spice_channel_get_type ())
 #define SPICE_CHANNEL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SPICE_TYPE_CHANNEL, SpiceChannel))
 #define SPICE_CHANNEL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SPICE_TYPE_CHANNEL, SpiceChannelClass))
@@ -131,9 +136,9 @@ gboolean spice_channel_test_common_capability(SpiceChannel *channel, guint32 cap
 void spice_channel_flush_async(SpiceChannel *channel, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data);
 gboolean spice_channel_flush_finish(SpiceChannel *channel, GAsyncResult *result, GError **error);
 #ifndef SPICE_DISABLE_DEPRECATED
-SPICE_DEPRECATED
+G_DEPRECATED
 void spice_channel_set_capability(SpiceChannel *channel, guint32 cap);
-SPICE_DEPRECATED
+G_DEPRECATED
 void spice_channel_destroy(SpiceChannel *channel);
 #endif
 
