@@ -38,6 +38,7 @@ struct stream {
     GstElement              *sink;
     guint                   rate;
     guint                   channels;
+    gboolean                fake; /* fake channel just for getting info about audio (volume) */
 };
 
 struct _SpiceGstaudioPrivate {
@@ -264,6 +265,8 @@ static gboolean update_mmtime_timeout_cb(gpointer data)
     SpiceGstaudioPrivate *p = gstaudio->priv;
     GstQuery *q;
 
+    g_return_val_if_fail(!p->playback.fake, TRUE);
+
     q = gst_query_new_latency();
     if (gst_element_query(p->playback.pipe, q)) {
         gboolean live;
@@ -326,7 +329,7 @@ cleanup:
     if (p->playback.pipe)
         gst_element_set_state(p->playback.pipe, GST_STATE_PLAYING);
 
-    if (p->mmtime_id == 0) {
+    if (!p->playback.fake && p->mmtime_id == 0) {
         update_mmtime_timeout_cb(gstaudio);
         p->mmtime_id = g_timeout_add_seconds(1, update_mmtime_timeout_cb, gstaudio);
     }
@@ -430,9 +433,6 @@ static void record_volume_changed(GObject *object, GParamSpec *pspec, gpointer d
     vol = 1.0 * volume[0] / VOLUME_NORMAL;
     SPICE_DEBUG("record volume changed to %u (%0.2f)", volume[0], 100*vol);
 
-    /* TODO directsoundsrc doesn't support IDirectSoundBuffer_SetVolume */
-    /* TODO pulsesrc doesn't support volume property, it's all coming! */
-
     if (GST_IS_BIN(p->record.src))
         e = gst_bin_get_by_interface(GST_BIN(p->record.src), GST_TYPE_STREAM_VOLUME);
     else
@@ -441,7 +441,7 @@ static void record_volume_changed(GObject *object, GParamSpec *pspec, gpointer d
     if (GST_IS_STREAM_VOLUME(e))
         gst_stream_volume_set_volume(GST_STREAM_VOLUME(e), GST_STREAM_VOLUME_FORMAT_CUBIC, vol);
     else
-        g_warning("gst lacks volume capabilities on src (TODO)");
+        g_warning("gst lacks volume capabilities on src");
 
     g_object_unref(e);
 }
@@ -467,7 +467,7 @@ static void record_mute_changed(GObject *object, GParamSpec *pspec, gpointer dat
     if (GST_IS_STREAM_VOLUME (e))
         gst_stream_volume_set_mute(GST_STREAM_VOLUME(e), mute);
     else
-        g_warning("gst lacks mute capabilities on src: %d (TODO)", mute);
+        g_warning("gst lacks mute capabilities on src: %d", mute);
 
     g_object_unref(e);
 }
@@ -572,7 +572,6 @@ static gboolean spice_gstaudio_get_playback_volume_info_finish(SpiceAudio *audio
     GstElement *e;
     gboolean lmute;
     gdouble vol;
-    gboolean fake_channel = FALSE;
     GTask *task = G_TASK(res);
 
     g_return_val_if_fail(g_task_is_valid(task, audio), FALSE);
@@ -587,9 +586,9 @@ static gboolean spice_gstaudio_get_playback_volume_info_finish(SpiceAudio *audio
 
     if (p->playback.sink == NULL || p->playback.channels == 0) {
         SPICE_DEBUG("PlaybackChannel not created yet, force start");
+        p->playback.fake = TRUE;
         /* In order to get system volume, we start the pipeline */
         playback_start(NULL, SPICE_AUDIO_FMT_S16, 2, 48000, audio);
-        fake_channel = TRUE;
     }
 
     if (GST_IS_BIN(p->playback.sink))
@@ -607,9 +606,10 @@ static gboolean spice_gstaudio_get_playback_volume_info_finish(SpiceAudio *audio
     }
     g_object_unref(e);
 
-    if (fake_channel) {
+    if (p->playback.fake) {
         SPICE_DEBUG("Stop faked PlaybackChannel");
         playback_stop(SPICE_GSTAUDIO(audio));
+        p->playback.fake = FALSE;
     }
 
     if (mute != NULL) {
