@@ -18,10 +18,10 @@
 #include "config.h"
 
 #include <glib.h>
+#include <gdk/gdk.h>
 
 #ifdef HAVE_X11_XKBLIB_H
 #include <X11/XKBlib.h>
-#include <gdk/gdkx.h>
 #endif
 #ifdef GDK_WINDOWING_X11
 #include <X11/Xlib.h>
@@ -126,6 +126,8 @@ static guint32 get_keyboard_lock_modifiers(void)
 {
     guint32 modifiers = 0;
 #if GTK_CHECK_VERSION(3,18,0)
+/* Ignore GLib's too-new warnings */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     GdkKeymap *keyboard = gdk_keymap_get_default();
 
     if (gdk_keymap_get_caps_lock_state(keyboard)) {
@@ -139,6 +141,7 @@ static guint32 get_keyboard_lock_modifiers(void)
     if (gdk_keymap_get_scroll_lock_state(keyboard)) {
         modifiers |= SPICE_INPUTS_SCROLL_LOCK;
     }
+G_GNUC_END_IGNORE_DEPRECATIONS
 #else
 #ifdef HAVE_X11_XKBLIB_H
     Display *x_display = NULL;
@@ -572,31 +575,26 @@ static const struct {
     }
 };
 
-typedef struct _WeakRef {
-    GObject *object;
-} WeakRef;
-
-static void weak_notify_cb(WeakRef *weakref, GObject *object)
+static GWeakRef* get_weak_ref(gpointer object)
 {
-    weakref->object = NULL;
-}
-
-static WeakRef* weak_ref(GObject *object)
-{
-    WeakRef *weakref = g_new(WeakRef, 1);
-
-    g_object_weak_ref(object, (GWeakNotify)weak_notify_cb, weakref);
-    weakref->object = object;
-
+    GWeakRef *weakref = g_new(GWeakRef, 1);
+    g_weak_ref_init(weakref, object);
     return weakref;
 }
 
-static void weak_unref(WeakRef* weakref)
+static gpointer free_weak_ref(gpointer data)
 {
-    if (weakref->object)
-        g_object_weak_unref(weakref->object, (GWeakNotify)weak_notify_cb, weakref);
+    GWeakRef *weakref = data;
+    gpointer object = g_weak_ref_get(weakref);
 
+    g_weak_ref_clear(weakref);
     g_free(weakref);
+    if (object != NULL) {
+        /* The main reference still exists as object is not NULL, so we can
+         * remove the strong reference given by g_weak_ref_get */
+        g_object_unref(object);
+    }
+    return object;
 }
 
 static void clipboard_get_targets(GtkClipboard *clipboard,
@@ -604,19 +602,24 @@ static void clipboard_get_targets(GtkClipboard *clipboard,
                                   gint n_atoms,
                                   gpointer user_data)
 {
-    WeakRef *weakref = user_data;
-    SpiceGtkSession *self = (SpiceGtkSession*)weakref->object;
-    weak_unref(weakref);
+    SpiceGtkSession *self = free_weak_ref(user_data);
+
+    SPICE_DEBUG("%s:", __FUNCTION__);
 
     if (self == NULL)
         return;
 
     g_return_if_fail(SPICE_IS_GTK_SESSION(self));
 
+    if (atoms == NULL) {
+        SPICE_DEBUG("Retrieving the clipboard data has failed");
+        return;
+    }
+
     SpiceGtkSessionPrivate *s = self->priv;
-    guint32 types[SPICE_N_ELEMENTS(atom2agent)];
-    char *name;
-    int a, m, t;
+    guint32 types[SPICE_N_ELEMENTS(atom2agent)] = { 0 };
+    gint num_types;
+    int a;
     int selection;
 
     if (s->main == NULL)
@@ -625,51 +628,54 @@ static void clipboard_get_targets(GtkClipboard *clipboard,
     selection = get_selection_from_clipboard(s, clipboard);
     g_return_if_fail(selection != -1);
 
-    SPICE_DEBUG("%s:", __FUNCTION__);
-    if (spice_util_get_debug()) {
-        for (a = 0; a < n_atoms; a++) {
-            name = gdk_atom_name(atoms[a]);
-            SPICE_DEBUG(" \"%s\"", name);
-            g_free(name);
-        }
+    if (s->clip_grabbed[selection]) {
+        SPICE_DEBUG("Clipboard is already grabbed, ignoring %d atoms", n_atoms);
+        return;
     }
 
-    memset(types, 0, sizeof(types));
+    /* Set all Atoms that matches our current protocol implementation */
+    num_types = 0;
     for (a = 0; a < n_atoms; a++) {
-        name = gdk_atom_name(atoms[a]);
+        guint m;
+        gchar *name = gdk_atom_name(atoms[a]);
+
+        SPICE_DEBUG(" \"%s\"", name);
+
         for (m = 0; m < SPICE_N_ELEMENTS(atom2agent); m++) {
+            guint t;
+
             if (strcasecmp(name, atom2agent[m].xatom) != 0) {
                 continue;
             }
-            /* found match */
-            for (t = 0; t < SPICE_N_ELEMENTS(atom2agent); t++) {
+
+            /* check if type is already in list */
+            for (t = 0; t < num_types; t++) {
                 if (types[t] == atom2agent[m].vdagent) {
-                    /* type already in list */
-                    break;
-                }
-                if (types[t] == 0) {
-                    /* add type to empty slot */
-                    types[t] = atom2agent[m].vdagent;
                     break;
                 }
             }
-            break;
+
+            if (t == num_types) {
+                /* add type to empty slot */
+                types[t] = atom2agent[m].vdagent;
+                num_types++;
+            }
         }
         g_free(name);
     }
-    for (t = 0; t < SPICE_N_ELEMENTS(atom2agent); t++) {
-        if (types[t] == 0) {
-            break;
-        }
-    }
-    if (!s->clip_grabbed[selection] && t > 0) {
-        s->clip_grabbed[selection] = TRUE;
 
-        if (spice_main_agent_test_capability(s->main, VD_AGENT_CAP_CLIPBOARD_BY_DEMAND))
-            spice_main_clipboard_selection_grab(s->main, selection, types, t);
-        /* Sending a grab causes the agent to do an impicit release */
-        s->nclip_targets[selection] = 0;
+    if (num_types == 0) {
+        SPICE_DEBUG("No GdkAtoms will be sent from %d", n_atoms);
+        return;
     }
+
+    s->clip_grabbed[selection] = TRUE;
+
+    if (spice_main_agent_test_capability(s->main, VD_AGENT_CAP_CLIPBOARD_BY_DEMAND))
+        spice_main_clipboard_selection_grab(s->main, selection, types, num_types);
+
+    /* Sending a grab causes the agent to do an implicit release */
+    s->nclip_targets[selection] = 0;
 }
 
 static void clipboard_owner_change(GtkClipboard        *clipboard,
@@ -703,7 +709,7 @@ static void clipboard_owner_change(GtkClipboard        *clipboard,
         s->clip_hasdata[selection] = TRUE;
         if (s->auto_clipboard_enable && !read_only(self))
             gtk_clipboard_request_targets(clipboard, clipboard_get_targets,
-                                          weak_ref(G_OBJECT(self)));
+                                          get_weak_ref(self));
         break;
     default:
         s->clip_hasdata[selection] = FALSE;
@@ -736,15 +742,7 @@ static void clipboard_got_from_guest(SpiceMainChannel *main, guint selection,
         /* on windows, gtk+ would already convert to LF endings, but
            not on unix */
         if (spice_main_agent_test_capability(s->main, VD_AGENT_CAP_GUEST_LINEEND_CRLF)) {
-            GError *err = NULL;
-
-            conv = spice_dos2unix((gchar*)data, size, &err);
-            if (err) {
-                g_warning("Failed to convert text line ending: %s", err->message);
-                g_clear_error(&err);
-                goto end;
-            }
-
+            conv = spice_dos2unix((gchar*)data, size);
             size = strlen(conv);
         }
 
@@ -755,7 +753,6 @@ static void clipboard_got_from_guest(SpiceMainChannel *main, guint selection,
             8, data, size);
     }
 
-end:
     if (g_main_loop_is_running (ri->loop))
         g_main_loop_quit (ri->loop);
 
@@ -814,11 +811,15 @@ static void clipboard_get(GtkClipboard *clipboard,
         goto cleanup;
     }
 
-    /* apparently, this is needed to avoid dead-lock, from
-       gtk_dialog_run */
+    /* This is modeled on the implementation of gtk_dialog_run() even though
+     * these thread functions are deprecated and appears to be needed to avoid
+     * dead-lock from gtk_dialog_run().
+     */
+    G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     gdk_threads_leave();
     g_main_loop_run(ri.loop);
     gdk_threads_enter();
+    G_GNUC_END_IGNORE_DEPRECATIONS
 
 cleanup:
     g_clear_pointer(&ri.loop, g_main_loop_unref);
@@ -910,13 +911,83 @@ static gboolean check_clipboard_size_limits(SpiceGtkSession *session,
     return TRUE;
 }
 
+/* This will convert line endings if needed (between Windows/Unix conventions),
+ * and will make sure 'len' does not take into account any trailing \0 as this could
+ * cause some confusion guest side.
+ * The 'len' argument will be modified by this function to the length of the modified
+ * string
+ */
+static char *fixup_clipboard_text(SpiceGtkSession *self, const char *text, int *len)
+{
+    char *conv = NULL;
+
+    if (spice_main_agent_test_capability(self->priv->main, VD_AGENT_CAP_GUEST_LINEEND_CRLF)) {
+        conv = spice_unix2dos(text, *len);
+        *len = strlen(conv);
+    } else {
+        /* On Windows, with some versions of gtk+, GtkSelectionData::length
+         * will include the final '\0'. When a string with this trailing '\0'
+         * is pasted in some linux applications, it will be pasted as <NIL> or
+         * as an invisible character, which is unwanted. Ensure the length we
+         * send to the agent does not include any trailing '\0'
+         * This is gtk+ bug https://bugzilla.gnome.org/show_bug.cgi?id=734670
+         */
+        *len = strlen(text);
+    }
+
+    return conv;
+}
+
+static void clipboard_received_text_cb(GtkClipboard *clipboard,
+                                       const gchar *text,
+                                       gpointer user_data)
+{
+    SpiceGtkSession *self = free_weak_ref(user_data);
+    char *conv = NULL;
+    int len = 0;
+    int selection;
+    const guchar *data = NULL;
+
+    if (self == NULL)
+        return;
+
+    selection = get_selection_from_clipboard(self->priv, clipboard);
+    g_return_if_fail(selection != -1);
+
+    if (text == NULL) {
+        SPICE_DEBUG("Failed to retrieve clipboard text");
+        goto notify_agent;
+    }
+
+    g_return_if_fail(SPICE_IS_GTK_SESSION(self));
+
+    len = strlen(text);
+    if (!check_clipboard_size_limits(self, len)) {
+        SPICE_DEBUG("Failed size limits of clipboard text (%d bytes)", len);
+        goto notify_agent;
+    }
+
+    /* gtk+ internal utf8 newline is always LF, even on windows */
+    conv = fixup_clipboard_text(self, text, &len);
+    if (!check_clipboard_size_limits(self, len)) {
+        SPICE_DEBUG("Failed size limits of clipboard text (%d bytes)", len);
+        goto notify_agent;
+    }
+
+    data = (const guchar *) (conv != NULL ? conv : text);
+notify_agent:
+    spice_main_clipboard_selection_notify(self->priv->main, selection,
+                                          VD_AGENT_CLIPBOARD_UTF8_TEXT,
+                                          data,
+                                          (data != NULL) ? len : 0);
+    g_free(conv);
+}
+
 static void clipboard_received_cb(GtkClipboard *clipboard,
                                   GtkSelectionData *selection_data,
                                   gpointer user_data)
 {
-    WeakRef *weakref = user_data;
-    SpiceGtkSession *self = (SpiceGtkSession*)weakref->object;
-    weak_unref(weakref);
+    SpiceGtkSession *self = free_weak_ref(user_data);
 
     if (self == NULL)
         return;
@@ -955,40 +1026,14 @@ static void clipboard_received_cb(GtkClipboard *clipboard,
     }
 
     const guchar *data = gtk_selection_data_get_data(selection_data);
-    gpointer conv = NULL;
 
-    /* gtk+ internal utf8 newline is always LF, even on windows */
-    if (type == VD_AGENT_CLIPBOARD_UTF8_TEXT) {
-        if (spice_main_agent_test_capability(s->main, VD_AGENT_CAP_GUEST_LINEEND_CRLF)) {
-            GError *err = NULL;
-
-            conv = spice_unix2dos((gchar*)data, len, &err);
-            if (err) {
-                g_warning("Failed to convert text line ending: %s", err->message);
-                g_clear_error(&err);
-                return;
-            }
-
-            len = strlen(conv);
-        } else {
-            /* On Windows, with some versions of gtk+, GtkSelectionData::length
-             * will include the final '\0'. When a string with this trailing '\0'
-             * is pasted in some linux applications, it will be pasted as <NIL> or
-             * as an invisible character, which is unwanted. Ensure the length we
-             * send to the agent does not include any trailing '\0'
-             * This is gtk+ bug https://bugzilla.gnome.org/show_bug.cgi?id=734670
-             */
-            len = strlen((const char *)data);
-        }
-        if (!check_clipboard_size_limits(self, len)) {
-            g_free(conv);
-            return;
-        }
-    }
+    /* text should be handled through clipboard_received_text_cb(), not
+     * clipboard_received_cb().
+     */
+    g_warn_if_fail(type != VD_AGENT_CLIPBOARD_UTF8_TEXT);
 
     spice_main_clipboard_selection_notify(s->main, selection, type,
-                                          conv ?: data, len);
-    g_free(conv);
+                                          data, len);
 }
 
 static gboolean clipboard_request(SpiceMainChannel *main, guint selection,
@@ -1011,17 +1056,22 @@ static gboolean clipboard_request(SpiceMainChannel *main, guint selection,
     cb = get_clipboard_from_selection(s, selection);
     g_return_val_if_fail(cb != NULL, FALSE);
 
-    for (m = 0; m < SPICE_N_ELEMENTS(atom2agent); m++) {
-        if (atom2agent[m].vdagent == type)
-            break;
+    if (type == VD_AGENT_CLIPBOARD_UTF8_TEXT) {
+        gtk_clipboard_request_text(cb, clipboard_received_text_cb,
+                                   get_weak_ref(self));
+    } else {
+        for (m = 0; m < SPICE_N_ELEMENTS(atom2agent); m++) {
+            if (atom2agent[m].vdagent == type)
+                break;
+        }
+
+        g_return_val_if_fail(m < SPICE_N_ELEMENTS(atom2agent), FALSE);
+
+        atom = gdk_atom_intern_static_string(atom2agent[m].xatom);
+        gtk_clipboard_request_contents(cb, atom, clipboard_received_cb,
+                                       get_weak_ref(self));
     }
 
-    g_return_val_if_fail(m < SPICE_N_ELEMENTS(atom2agent), FALSE);
-
-    atom = gdk_atom_intern_static_string(atom2agent[m].xatom);
-    gtk_clipboard_request_contents(cb, atom, clipboard_received_cb,
-                                   weak_ref(G_OBJECT(self)));
-
     return TRUE;
 }
 
@@ -1192,7 +1242,7 @@ void spice_gtk_session_copy_to_guest(SpiceGtkSession *self)
 
     if (s->clip_hasdata[selection] && !s->clip_grabbed[selection]) {
         gtk_clipboard_request_targets(s->clipboard, clipboard_get_targets,
-                                      weak_ref(G_OBJECT(self)));
+                                      get_weak_ref(self));
     }
 }
 
