@@ -35,7 +35,6 @@
 #include <gudev/gudev.h>
 #elif defined(G_OS_WIN32)
 #include "win-usb-dev.h"
-#include "win-usb-driver-install.h"
 #define USE_GUDEV /* win-usb-dev.h provides a fake gudev interface */
 #elif !defined USE_LIBUSB_HOTPLUG
 #error "Expecting one of USE_GUDEV or USE_LIBUSB_HOTPLUG to be defined"
@@ -51,7 +50,7 @@
 #include "spice-marshal.h"
 #include "usb-device-manager-priv.h"
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #ifndef G_OS_WIN32 /* Linux -- device id is bus.addr */
 #define DEV_ID_FMT "at %u.%u"
@@ -112,7 +111,7 @@ struct _SpiceUsbDeviceManagerPrivate {
     libusb_context *context;
     int event_listeners;
     GThread *event_thread;
-    gboolean event_thread_run;
+    gint event_thread_run;
     struct usbredirfilter_rule *auto_conn_filter_rules;
     struct usbredirfilter_rule *redirect_on_connect_rules;
     int auto_conn_filter_rules_count;
@@ -127,9 +126,7 @@ struct _SpiceUsbDeviceManagerPrivate {
 #ifdef G_OS_WIN32
     usbdk_api_wrapper     *usbdk_api;
     HANDLE                 usbdk_hider_handle;
-    SpiceWinUsbDriver     *installer;
 #endif
-    gboolean               use_usbclerk;
 #endif
     GPtrArray *devices;
     GPtrArray *channels;
@@ -153,6 +150,7 @@ typedef struct _SpiceUsbDeviceInfo {
     guint8  devaddr;
     guint16 vid;
     guint16 pid;
+    gboolean isochronous;
 #ifdef G_OS_WIN32
     guint8  state;
 #else
@@ -187,9 +185,6 @@ static SpiceUsbDevice *spice_usb_device_ref(SpiceUsbDevice *device);
 static void spice_usb_device_unref(SpiceUsbDevice *device);
 
 #ifdef G_OS_WIN32
-static guint8 spice_usb_device_get_state(SpiceUsbDevice *device);
-static void  spice_usb_device_set_state(SpiceUsbDevice *device, guint8 s);
-
 static void _usbdk_hider_update(SpiceUsbDeviceManager *manager);
 static void _usbdk_hider_clear(SpiceUsbDeviceManager *manager);
 #endif
@@ -212,6 +207,9 @@ static
 void _connect_device_async_cb(GObject *gobject,
                               GAsyncResult *channel_res,
                               gpointer user_data);
+static
+void disconnect_device_sync(SpiceUsbDeviceManager *self,
+                            SpiceUsbDevice *device);
 
 G_DEFINE_BOXED_TYPE(SpiceUsbDevice, spice_usb_device,
                     (GBoxedCopyFunc)spice_usb_device_ref,
@@ -273,8 +271,11 @@ static void spice_usb_device_manager_init(SpiceUsbDeviceManager *self)
     self->priv = priv;
 
 #if defined(G_OS_WIN32) && defined(USE_USBREDIR)
-    priv->use_usbclerk = !usbdk_is_driver_installed() ||
-                         !(priv->usbdk_api = usbdk_api_load());
+    if (usbdk_is_driver_installed()) {
+        priv->usbdk_api = usbdk_api_load();
+    } else {
+        spice_debug("UsbDk driver is not installed");
+    }
 #endif
     priv->channels = g_ptr_array_new();
 #ifdef USE_USBREDIR
@@ -297,16 +298,6 @@ static gboolean spice_usb_device_manager_initable_init(GInitable  *initable,
     const gchar *const subsystems[] = {"usb", NULL};
 #endif
 
-#ifdef G_OS_WIN32
-    if (priv->use_usbclerk) {
-        priv->installer = spice_win_usb_driver_new(err);
-        if (!priv->installer) {
-            SPICE_DEBUG("failed to initialize winusb driver");
-            return FALSE;
-        }
-    }
-#endif
-
     /* Initialize libusb */
     rc = libusb_init(&priv->context);
     if (rc < 0) {
@@ -320,6 +311,10 @@ static gboolean spice_usb_device_manager_initable_init(GInitable  *initable,
     /* Start listening for usb devices plug / unplug */
 #ifdef USE_GUDEV
     priv->udev = g_udev_client_new(subsystems);
+    if (priv->udev == NULL) {
+        g_warning("Error initializing GUdevClient");
+        return FALSE;
+    }
     g_signal_connect(G_OBJECT(priv->udev), "uevent",
                      G_CALLBACK(spice_usb_device_manager_uevent_cb), self);
     /* Do coldplug (detection of already connected devices) */
@@ -375,12 +370,22 @@ static void spice_usb_device_manager_dispose(GObject *gobject)
 #ifdef USE_LIBUSB_HOTPLUG
     if (priv->hp_handle) {
         spice_usb_device_manager_stop_event_listening(self);
+        if (g_atomic_int_get(&priv->event_thread_run)) {
+            /* Force termination of the event thread even if there were some
+             * mismatched spice_usb_device_manager_{start,stop}_event_listening
+             * calls. Otherwise, the usb event thread will be leaked, and will
+             * try to use the libusb context we destroy in finalize(), which would
+             * cause a crash */
+             g_warn_if_reached();
+             g_atomic_int_set(&priv->event_thread_run, FALSE);
+        }
         /* This also wakes up the libusb_handle_events() in the event_thread */
         libusb_hotplug_deregister_callback(priv->context, priv->hp_handle);
         priv->hp_handle = 0;
     }
 #endif
-    if (priv->event_thread && !priv->event_thread_run) {
+    if (priv->event_thread) {
+        g_warn_if_fail(g_atomic_int_get(&priv->event_thread_run) == FALSE);
         g_thread_join(priv->event_thread);
         priv->event_thread = NULL;
     }
@@ -410,14 +415,8 @@ static void spice_usb_device_manager_finalize(GObject *gobject)
     free(priv->auto_conn_filter_rules);
     free(priv->redirect_on_connect_rules);
 #ifdef G_OS_WIN32
-    if (priv->installer) {
-        g_warn_if_fail(priv->use_usbclerk);
-        g_object_unref(priv->installer);
-    }
-    if (!priv->use_usbclerk) {
-        _usbdk_hider_clear(self);
-        usbdk_api_unload(priv->usbdk_api);
-    }
+    _usbdk_hider_clear(self);
+    usbdk_api_unload(priv->usbdk_api);
 #endif
 #endif
 
@@ -490,9 +489,7 @@ static void spice_usb_device_manager_set_property(GObject       *gobject,
     case PROP_AUTO_CONNECT:
         priv->auto_connect = g_value_get_boolean(value);
 #if defined(G_OS_WIN32) && defined(USE_USBREDIR)
-        if (!priv->use_usbclerk) {
-            _usbdk_hider_update(self);
-        }
+        _usbdk_hider_update(self);
 #endif
         break;
     case PROP_AUTO_CONNECT_FILTER: {
@@ -518,9 +515,7 @@ static void spice_usb_device_manager_set_property(GObject       *gobject,
         priv->auto_connect_filter = g_strdup(filter);
 
 #if defined(G_OS_WIN32) && defined(USE_USBREDIR)
-        if (!priv->use_usbclerk) {
-            _usbdk_hider_update(self);
-        }
+        _usbdk_hider_update(self);
 #endif
         break;
     }
@@ -640,9 +635,9 @@ static void spice_usb_device_manager_class_init(SpiceUsbDeviceManagerClass *klas
                                     pspec);
 
     /**
-     * SpiceUsbDeviceManager:n-free-channels:
+     * SpiceUsbDeviceManager:free-channels:
      *
-     * Get a list of avaialable channels for redirecting USB devices.
+     * Get the number of available channels for redirecting USB devices.
      *
      * Since: 0.31
      */
@@ -754,15 +749,10 @@ static gboolean spice_usb_device_manager_get_udev_bus_n_address(
 
     *bus = *address = 0;
 
-    if (manager->priv->use_usbclerk) {
-       /* Windows WinUsb/UsbClerk -- request vid:pid instead */
-        bus_str = g_udev_device_get_property(udev, "VID");
-        address_str = g_udev_device_get_property(udev, "PID");
-    } else {
-       /* Linux or UsbDk backend on Windows*/
-        bus_str = g_udev_device_get_property(udev, "BUSNUM");
-        address_str = g_udev_device_get_property(udev, "DEVNUM");
-    }
+   /* Linux or UsbDk backend on Windows*/
+    bus_str = g_udev_device_get_property(udev, "BUSNUM");
+    address_str = g_udev_device_get_property(udev, "DEVNUM");
+
     if (bus_str)
         *bus = atoi(bus_str);
     if (address_str)
@@ -796,6 +786,7 @@ static gboolean spice_usb_device_manager_get_device_descriptor(
     return TRUE;
 }
 
+#endif // USE_USBREDIR
 
 /**
  * spice_usb_device_get_libusb_device:
@@ -822,6 +813,7 @@ spice_usb_device_get_libusb_device(const SpiceUsbDevice *device G_GNUC_UNUSED)
     return NULL;
 }
 
+#ifdef USE_USBREDIR
 static gboolean spice_usb_device_manager_get_libdev_vid_pid(
     libusb_device *libdev, int *vid, int *pid)
 {
@@ -905,13 +897,8 @@ static gboolean
 spice_usb_device_manager_device_match(SpiceUsbDeviceManager *self, SpiceUsbDevice *device,
                                       const int bus, const int address)
 {
-   if (self->priv->use_usbclerk) {
-        return (spice_usb_device_get_vid(device) == bus &&
-                spice_usb_device_get_pid(device) == address);
-    } else {
-        return (spice_usb_device_get_busnum(device) == bus &&
-                spice_usb_device_get_devaddr(device) == address);
-    }
+    return (spice_usb_device_get_busnum(device) == bus &&
+            spice_usb_device_get_devaddr(device) == address);
 }
 
 #ifdef USE_GUDEV
@@ -919,19 +906,9 @@ static gboolean
 spice_usb_device_manager_libdev_match(SpiceUsbDeviceManager *self, libusb_device *libdev,
                                       const int bus, const int address)
 {
-    if (self->priv->use_usbclerk) {
-        /* WinUSB -- match functions for Windows -- match by vid:pid */
-        int vid, pid;
-
-        if (!spice_usb_device_manager_get_libdev_vid_pid(libdev, &vid, &pid)) {
-            return FALSE;
-        }
-        return (bus == vid && address == pid);
-    } else {
-        /* match functions for Linux/UsbDk -- match by bus.addr */
-        return (libusb_get_bus_number(libdev) == bus &&
-                libusb_get_device_address(libdev) == address);
-    }
+    /* match functions for Linux/UsbDk -- match by bus.addr */
+    return (libusb_get_bus_number(libdev) == bus &&
+            libusb_get_device_address(libdev) == address);
 }
 #endif
 
@@ -1011,19 +988,8 @@ static void spice_usb_device_manager_remove_dev(SpiceUsbDeviceManager *self,
         return;
     }
 
-#ifdef G_OS_WIN32
-    if (priv->use_usbclerk) {
-        const guint8 state = spice_usb_device_get_state(device);
-        if ((state == SPICE_USB_DEVICE_STATE_INSTALLING) ||
-            (state == SPICE_USB_DEVICE_STATE_UNINSTALLING)) {
-            SPICE_DEBUG("skipping " DEV_ID_FMT ". It is un/installing its driver",
-                        bus, address);
-            return;
-        }
-    }
-#endif
-
-    spice_usb_device_manager_disconnect_device(self, device);
+    /* TODO: check usage of the sync version is ok here */
+    disconnect_device_sync(self, device);
 
     SPICE_DEBUG("device removed %04x:%04x (%p)",
                 spice_usb_device_get_vid(device),
@@ -1153,7 +1119,7 @@ static int spice_usb_device_manager_hotplug_cb(libusb_context       *ctx,
     g_idle_add(spice_usb_device_manager_hotplug_idle_cb, args);
     return 0;
 }
-#endif
+#endif // USE_USBREDIR
 
 static void spice_usb_device_manager_channel_connect_cb(
     GObject *gobject, GAsyncResult *channel_res, gpointer user_data)
@@ -1171,106 +1137,6 @@ static void spice_usb_device_manager_channel_connect_cb(
     g_object_unref(task);
 }
 
-#ifdef G_OS_WIN32
-
-typedef struct _UsbInstallCbInfo {
-    SpiceUsbDeviceManager *manager;
-    SpiceUsbDevice        *device;
-    SpiceWinUsbDriver     *installer;
-    GCancellable          *cancellable;
-    GAsyncReadyCallback   callback;
-    gpointer              user_data;
-} UsbInstallCbInfo;
-
-/**
- * spice_usb_device_manager_drv_install_cb:
- * @gobject: #SpiceWinUsbDriver in charge of installing the driver
- * @res: #GAsyncResult of async win usb driver installation
- * @user_data: #SpiceUsbDeviceManager requested the installation
- *
- * Called when an Windows libusb driver installation completed.
- *
- * If the driver installation was successful, continue with USB
- * device redirection
- *
- * Always call _spice_usb_device_manager_connect_device_async.
- * When installation fails, libusb_open fails too, but cleanup would be better.
- */
-static void spice_usb_device_manager_drv_install_cb(GObject *gobject,
-                                                    GAsyncResult *res,
-                                                    gpointer user_data)
-{
-    SpiceUsbDeviceManager *self;
-    SpiceWinUsbDriver *installer;
-    GError *err = NULL;
-    SpiceUsbDevice *device;
-    UsbInstallCbInfo *cbinfo;
-    GCancellable *cancellable;
-    gpointer data;
-
-    g_return_if_fail(user_data != NULL);
-
-    cbinfo = user_data;
-    self        = cbinfo->manager;
-    device      = cbinfo->device;
-    installer   = cbinfo->installer;
-    cancellable = cbinfo->cancellable;
-    data        = cbinfo->user_data;
-
-    g_free(cbinfo);
-
-    g_return_if_fail(SPICE_IS_USB_DEVICE_MANAGER(self));
-    g_return_if_fail(self->priv->use_usbclerk);
-    g_return_if_fail(SPICE_IS_WIN_USB_DRIVER(installer));
-    g_return_if_fail(device!= NULL);
-
-    SPICE_DEBUG("Win USB driver install finished");
-
-    if (!spice_win_usb_driver_install_finish(installer, res, &err)) {
-        g_warning("win usb driver install failed -- %s", err->message);
-        g_error_free(err);
-    }
-
-    spice_usb_device_set_state(device, SPICE_USB_DEVICE_STATE_INSTALLED);
-
-    /* device is already ref'ed */
-    _spice_usb_device_manager_connect_device_async(self,
-                                                   device,
-                                                   cancellable,
-                                                   _connect_device_async_cb,
-                                                   data);
-
-    spice_usb_device_unref(device);
-}
-
-static void spice_usb_device_manager_drv_uninstall_cb(GObject *gobject,
-                                                      GAsyncResult *res,
-                                                      gpointer user_data)
-{
-    GError *err = NULL;
-
-    UsbInstallCbInfo *cbinfo = user_data;
-    SpiceUsbDeviceManager *self = cbinfo->manager;
-    SpiceUsbDevice *device = cbinfo->device;
-    SpiceWinUsbDriver *installer = cbinfo->installer;
-
-    g_free(cbinfo);
-
-    SPICE_DEBUG("Win USB driver uninstall finished");
-    g_return_if_fail(SPICE_IS_USB_DEVICE_MANAGER(self));
-    g_return_if_fail(self->priv->use_usbclerk);
-
-    if (!spice_win_usb_driver_uninstall_finish(installer, res, &err)) {
-        g_warning("win usb driver uninstall failed -- %s", err->message);
-        g_clear_error(&err);
-    }
-
-    spice_usb_device_set_state(device, SPICE_USB_DEVICE_STATE_NONE);
-    spice_usb_device_unref(device);
-}
-
-#endif
-
 /* ------------------------------------------------------------------ */
 /* private api                                                        */
 
@@ -1280,7 +1146,7 @@ static gpointer spice_usb_device_manager_usb_ev_thread(gpointer user_data)
     SpiceUsbDeviceManagerPrivate *priv = self->priv;
     int rc;
 
-    while (priv->event_thread_run) {
+    while (g_atomic_int_get(&priv->event_thread_run)) {
         rc = libusb_handle_events(priv->context);
         if (rc && rc != LIBUSB_ERROR_INTERRUPTED) {
             const char *desc = spice_usbutil_libusb_strerror(rc);
@@ -1310,7 +1176,7 @@ gboolean spice_usb_device_manager_start_event_listening(
          g_thread_join(priv->event_thread);
          priv->event_thread = NULL;
     }
-    priv->event_thread_run = TRUE;
+    g_atomic_int_set(&priv->event_thread_run, TRUE);
     priv->event_thread = g_thread_new("usb_ev_thread",
                                       spice_usb_device_manager_usb_ev_thread,
                                       self);
@@ -1326,7 +1192,7 @@ void spice_usb_device_manager_stop_event_listening(
 
     priv->event_listeners--;
     if (priv->event_listeners == 0)
-        priv->event_thread_run = FALSE;
+        g_atomic_int_set(&priv->event_thread_run, FALSE);
 }
 
 static void spice_usb_device_manager_check_redir_on_connect(
@@ -1433,7 +1299,7 @@ GPtrArray* spice_usb_device_manager_get_devices_with_filter(
 
 #ifdef USE_USBREDIR
     SpiceUsbDeviceManagerPrivate *priv = self->priv;
-    struct usbredirfilter_rule *rules = NULL;;
+    struct usbredirfilter_rule *rules = NULL;
     int r, count = 0;
     guint i;
 
@@ -1503,68 +1369,6 @@ gboolean spice_usb_device_manager_is_device_connected(SpiceUsbDeviceManager *sel
     return !!spice_usb_device_manager_get_channel_for_dev(self, device);
 }
 
-#if defined(USE_USBREDIR) && defined(G_OS_WIN32)
-
-static void
-_spice_usb_device_manager_install_driver_async(SpiceUsbDeviceManager *self,
-                                               SpiceUsbDevice *device,
-                                               GCancellable *cancellable,
-                                               GAsyncReadyCallback callback,
-                                               gpointer user_data)
-{
-    SpiceWinUsbDriver *installer;
-    UsbInstallCbInfo *cbinfo;
-
-    g_return_if_fail(self->priv->installer);
-
-    spice_usb_device_set_state(device, SPICE_USB_DEVICE_STATE_INSTALLING);
-
-    installer = self->priv->installer;
-    cbinfo = g_new0(UsbInstallCbInfo, 1);
-    cbinfo->manager     = self;
-    cbinfo->device      = spice_usb_device_ref(device);
-    cbinfo->installer   = installer;
-    cbinfo->cancellable = cancellable;
-    cbinfo->callback    = callback;
-    cbinfo->user_data   = user_data;
-
-    spice_win_usb_driver_install_async(installer, device, cancellable,
-                                       spice_usb_device_manager_drv_install_cb,
-                                       cbinfo);
-}
-
-static void
-_spice_usb_device_manager_uninstall_driver_async(SpiceUsbDeviceManager *self,
-                                                 SpiceUsbDevice *device)
-{
-    SpiceWinUsbDriver *installer;
-    UsbInstallCbInfo *cbinfo;
-    guint8 state;
-
-    g_warn_if_fail(device != NULL);
-    g_return_if_fail(self->priv->installer);
-
-    state = spice_usb_device_get_state(device);
-    if ((state != SPICE_USB_DEVICE_STATE_INSTALLED) &&
-        (state != SPICE_USB_DEVICE_STATE_CONNECTED)) {
-        return;
-    }
-
-    spice_usb_device_set_state(device, SPICE_USB_DEVICE_STATE_UNINSTALLING);
-
-    installer = self->priv->installer;
-    cbinfo = g_new0(UsbInstallCbInfo, 1);
-    cbinfo->manager     = self;
-    cbinfo->device      = spice_usb_device_ref(device);
-    cbinfo->installer   = installer;
-
-    spice_win_usb_driver_uninstall_async(installer, device, NULL,
-                                         spice_usb_device_manager_drv_uninstall_cb,
-                                         cbinfo);
-}
-
-#endif
-
 #ifdef USE_USBREDIR
 
 static gboolean
@@ -1676,21 +1480,11 @@ void spice_usb_device_manager_connect_device_async(SpiceUsbDeviceManager *self,
         g_task_new(G_OBJECT(self), cancellable, callback, user_data);
 
     _set_redirecting(self, TRUE);
-
-#ifdef G_OS_WIN32
-    if (self->priv->use_usbclerk) {
-        _spice_usb_device_manager_install_driver_async(self, device, cancellable,
-                                                       callback, task);
-        return;
-    }
-#endif
-
     _spice_usb_device_manager_connect_device_async(self,
                                                    device,
                                                    cancellable,
                                                    _connect_device_async_cb,
                                                    task);
-
 #endif
 }
 
@@ -1754,15 +1548,8 @@ void _connect_device_async_cb(GObject *gobject,
 }
 #endif
 
-/**
- * spice_usb_device_manager_disconnect_device:
- * @manager: the #SpiceUsbDeviceManager manager
- * @device: a #SpiceUsbDevice to disconnect
- *
- * Disconnects the @device.
- */
-void spice_usb_device_manager_disconnect_device(SpiceUsbDeviceManager *self,
-                                                SpiceUsbDevice *device)
+static void disconnect_device_sync(SpiceUsbDeviceManager *self,
+                                   SpiceUsbDevice *device)
 {
     g_return_if_fail(SPICE_IS_USB_DEVICE_MANAGER(self));
     g_return_if_fail(device != NULL);
@@ -1776,12 +1563,20 @@ void spice_usb_device_manager_disconnect_device(SpiceUsbDeviceManager *self,
     if (channel)
         spice_usbredir_channel_disconnect_device(channel);
 
-#ifdef G_OS_WIN32
-    if(self->priv->use_usbclerk)
-        _spice_usb_device_manager_uninstall_driver_async(self, device);
 #endif
+}
 
-#endif
+/**
+ * spice_usb_device_manager_disconnect_device:
+ * @manager: the #SpiceUsbDeviceManager manager
+ * @device: a #SpiceUsbDevice to disconnect
+ *
+ * Disconnects the @device.
+ */
+void spice_usb_device_manager_disconnect_device(SpiceUsbDeviceManager *self,
+                                                SpiceUsbDevice *device)
+{
+    disconnect_device_sync(self, device);
 }
 
 typedef struct _disconnect_cb_data
@@ -1802,12 +1597,6 @@ void _disconnect_device_async_cb(GObject *gobject,
     disconnect_cb_data *data = g_task_get_task_data(task);
     SpiceUsbDeviceManager *self = SPICE_USB_DEVICE_MANAGER(data->self);
 
-#ifdef G_OS_WIN32
-    if (self->priv->use_usbclerk) {
-        _spice_usb_device_manager_uninstall_driver_async(self, data->device);
-    }
-#endif
-
     _set_redirecting(self, FALSE);
 
     spice_usbredir_channel_disconnect_device_finish(channel, channel_res, &err);
@@ -1820,6 +1609,20 @@ void _disconnect_device_async_cb(GObject *gobject,
 }
 #endif
 
+/**
+ * spice_usb_device_manager_disconnect_device_async:
+ * @self: the #SpiceUsbDeviceManager manager.
+ * @device: a connected #SpiceUsbDevice to disconnect.
+ * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
+ * @callback: (scope async): a #GAsyncReadyCallback to call when the request is satisfied.
+ * @user_data: (closure): the data to pass to @callback.
+ *
+ * Asynchronously disconnects the @device. When completed, @callback will be called.
+ * Then it is possible to call spice_usb_device_manager_disconnect_device_finish()
+ * to get the result of the operation.
+ *
+ * Since: 0.32
+ */
 void spice_usb_device_manager_disconnect_device_async(SpiceUsbDeviceManager *self,
                                                       SpiceUsbDevice *device,
                                                       GCancellable *cancellable,
@@ -2007,6 +1810,33 @@ gchar *spice_usb_device_get_description(SpiceUsbDevice *device, const gchar *for
 
 
 #ifdef USE_USBREDIR
+static gboolean probe_isochronous_endpoint(libusb_device *libdev)
+{
+    struct libusb_config_descriptor *conf_desc;
+    gboolean isoc_found = FALSE;
+    gint i, j, k;
+
+    g_return_val_if_fail(libdev != NULL, FALSE);
+
+    if (libusb_get_active_config_descriptor(libdev, &conf_desc) != 0) {
+        g_return_val_if_reached(FALSE);
+    }
+
+    for (i = 0; !isoc_found && i < conf_desc->bNumInterfaces; i++) {
+        for (j = 0; !isoc_found && j < conf_desc->interface[i].num_altsetting; j++) {
+            for (k = 0; !isoc_found && k < conf_desc->interface[i].altsetting[j].bNumEndpoints;k++) {
+                gint attributes = conf_desc->interface[i].altsetting[j].endpoint[k].bmAttributes;
+                gint type = attributes & LIBUSB_TRANSFER_TYPE_MASK;
+                if (type == LIBUSB_TRANSFER_TYPE_ISOCHRONOUS)
+                    isoc_found = TRUE;
+            }
+        }
+    }
+
+    libusb_free_config_descriptor(conf_desc);
+    return isoc_found;
+}
+
 /*
  * SpiceUsbDeviceInfo
  */
@@ -2032,6 +1862,7 @@ static SpiceUsbDeviceInfo *spice_usb_device_new(libusb_device *libdev)
     info->vid = vid;
     info->pid = pid;
     info->ref = 1;
+    info->isochronous = probe_isochronous_endpoint(libdev);
 #ifndef G_OS_WIN32
     info->libdev = libusb_ref_device(libdev);
 #endif
@@ -2075,31 +1906,22 @@ guint16 spice_usb_device_get_pid(const SpiceUsbDevice *device)
     return info->pid;
 }
 
-#ifdef G_OS_WIN32
-void spice_usb_device_set_state(SpiceUsbDevice *device, guint8 state)
+gboolean spice_usb_device_is_isochronous(const SpiceUsbDevice *device)
 {
-    SpiceUsbDeviceInfo *info = (SpiceUsbDeviceInfo *)device;
-
-    g_return_if_fail(info != NULL);
-
-    info->state = state;
-}
-
-guint8 spice_usb_device_get_state(SpiceUsbDevice *device)
-{
-    SpiceUsbDeviceInfo *info = (SpiceUsbDeviceInfo *)device;
+    const SpiceUsbDeviceInfo *info = (const SpiceUsbDeviceInfo *)device;
 
     g_return_val_if_fail(info != NULL, 0);
 
-    return info->state;
+    return info->isochronous;
 }
 
+#ifdef G_OS_WIN32
 static
 gboolean _usbdk_hider_prepare(SpiceUsbDeviceManager *manager)
 {
     SpiceUsbDeviceManagerPrivate *priv = manager->priv;
 
-    g_return_val_if_fail(!priv->use_usbclerk, FALSE);
+    g_return_val_if_fail(priv->usbdk_api != NULL, FALSE);
 
     if (priv->usbdk_hider_handle == NULL) {
         priv->usbdk_hider_handle = usbdk_create_hider_handle(priv->usbdk_api);
@@ -2117,7 +1939,7 @@ void _usbdk_hider_clear(SpiceUsbDeviceManager *manager)
 {
     SpiceUsbDeviceManagerPrivate *priv = manager->priv;
 
-    g_return_if_fail(!priv->use_usbclerk);
+    g_return_if_fail(priv->usbdk_api != NULL);
 
     if (priv->usbdk_hider_handle != NULL) {
         usbdk_clear_hide_rules(priv->usbdk_api, priv->usbdk_hider_handle);
@@ -2131,7 +1953,7 @@ void _usbdk_hider_update(SpiceUsbDeviceManager *manager)
 {
     SpiceUsbDeviceManagerPrivate *priv = manager->priv;
 
-    g_return_if_fail(!priv->use_usbclerk);
+    g_return_if_fail(priv->usbdk_api != NULL);
 
     if (priv->auto_connect_filter == NULL) {
         SPICE_DEBUG("No autoredirect rules, no hider setup needed");
@@ -2139,6 +1961,12 @@ void _usbdk_hider_update(SpiceUsbDeviceManager *manager)
         return;
     }
 
+    if (!spice_session_get_usbredir_enabled(priv->session)) {
+        SPICE_DEBUG("USB redirection disabled, no hider setup needed");
+        _usbdk_hider_clear(manager);
+        return;
+    }
+
     if (!priv->auto_connect) {
         SPICE_DEBUG("Auto-connect disabled, no hider setup needed");
         _usbdk_hider_clear(manager);
@@ -2204,14 +2032,8 @@ spice_usb_manager_device_equal_libdev(SpiceUsbDeviceManager *manager,
     if ((device == NULL) || (libdev == NULL))
         return FALSE;
 
-    if (manager->priv->use_usbclerk) {
-        busnum = spice_usb_device_get_vid(device);
-        devaddr = spice_usb_device_get_pid(device);
-    } else {
-        busnum = spice_usb_device_get_busnum(device);
-        devaddr = spice_usb_device_get_devaddr(device);
-    }
-
+    busnum = spice_usb_device_get_busnum(device);
+    devaddr = spice_usb_device_get_devaddr(device);
     return spice_usb_device_manager_libdev_match(manager, libdev,
                                                  busnum, devaddr);
 }
