@@ -20,7 +20,7 @@
 */
 
 #include "config.h"
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "spice-client.h"
 #include "spice-marshal.h"
 #include "usb-device-widget.h"
@@ -40,14 +40,14 @@
 
 struct _SpiceUsbDeviceWidget
 {
-    GtkVBox parent;
+    GtkBox parent;
 
     SpiceUsbDeviceWidgetPrivate *priv;
 };
 
 struct _SpiceUsbDeviceWidgetClass
 {
-    GtkVBoxClass parent_class;
+    GtkBoxClass parent_class;
 
     /* signals */
     void (*connect_failed) (SpiceUsbDeviceWidget *widget,
